<?php

/** 
 * Joomla.Plugin - System - NetzSinn Custom Fields Filter
 * ------------------------------------------------------------------------
 * @package     System - NetzSinn Custom Fields Filter
 * @author      Netzsinn.de
 * @copyright   2023 netzsinn.de
 * @license     GNU/GPLv3 <http://www.gnu.org/licenses/gpl-3.0.de.html>
 * @link        https://netzsinn.de
 * ------------------------------------------------------------------------
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;

class NetzSinnCustomfieldsFieldsHelper
{
    public static function resolveCategory($catname, $fieldid)
    {
        $app = Factory::getApplication();
        $params =  $app->getUserState('polymorphart.customfieldsfilter.params');

        $db = Factory::getDbo();
        $query = $db->getQuery(true);
        $query->select('fieldparams');
        $query->from($db->quoteName('#__fields'));
        $query->where($db->quoteName('id') . " = " . $db->quote($fieldid));

        try {
            if ($params->get('is_debug') == true && !Factory::getUser()->guest) {
                echo '<pre>';
                echo $query;
                echo '</pre>';
            }
            $db->setQuery($query);
            $result = $db->loadObject();
        } catch (Exception $e) {
            $msg = $e->getMessage(); // Returns "Normally you would have other code...
            $code = $e->getCode(); // Returns '500';
            Factory::getApplication()->enqueueMessage($msg, 'error'); // commonly to still display that error
            return false;
        }
        $fieldparams = json_decode($result->fieldparams);
        #print_r($fieldparams);

        $result = new stdClass;
        foreach ($fieldparams->options as $option) {
            if (strtolower(str_replace(' ', '-', $option->name)) == strtolower(trim($catname))) {
                $result->id = $option->value;
                $result->name = $option->name;
                break;
            }
        }
        return $result;
    }

    public static function resolveTyp($catname, $fieldid)
    {
        #echo 'catname: ' . $catname . '<br/>';
        #echo 'fieldid: ' . $fieldid . '<br/>';
        $app = Factory::getApplication();
        $is_debug =  $app->getUserState('netzsinn.customfieldsfilter.debug');

        $db = Factory::getDbo();
        $query = $db->getQuery(true);
        $query->select('fieldparams');
        $query->from($db->quoteName('#__fields'));
        $query->where($db->quoteName('id') . " = " . $db->quote($fieldid));

        try {
            if ($is_debug == true && !Factory::getUser()->guest) {
                echo '<pre>';
                echo $query;
                echo '</pre>';
            }
            $db->setQuery($query);
            $result = $db->loadObject();
        } catch (Exception $e) {
            $msg = $e->getMessage(); // Returns "Normally you would have other code...
            $code = $e->getCode(); // Returns '500';
            Factory::getApplication()->enqueueMessage($msg, 'error'); // commonly to still display that error
            return false;
        }
        $fieldparams = json_decode($result->fieldparams);
        #print_r($fieldparams);
        #echo $fieldid;
        #echo $catname;

        $result = new stdClass;
        foreach ($fieldparams->options as $option) {

            if (strpos(strtolower(trim($catname)), strtolower($option->name)) !== false) {
                $result->id = $option->value;
                $result->name = $option->name;
                break;
            }
        }
        return $result;
    }

    public static function resolveTag($fieldvalue, $catid)
    {
        $app = Factory::getApplication();
        $is_debug =  $app->getUserState('netzsinn.customfieldsfilter.debug');

        $db = Factory::getDbo();
        $query = $db->getQuery(true);
        $query->select('id, title');
        $query->from($db->quoteName('#__content'));
        $query->where($db->quoteName(strtolower(trim('alias'))) . " = " . $db->quote(strtolower(trim($fieldvalue))));
        $query->where($db->quoteName('catid') . " = " . $db->quote($catid));

        try {
            if ($is_debug == true && !Factory::getUser()->guest) {
                echo '<pre>';
                echo $query;
                echo '</pre>';
            }
            $db->setQuery($query);
            $result = $db->loadObject();
        } catch (Exception $e) {
            $msg = $e->getMessage(); // Returns "Normally you would have other code...
            $code = $e->getCode(); // Returns '500';
            Factory::getApplication()->enqueueMessage($msg, 'error'); // commonly to still display that error
            return false;
        }
        return $result;
    }

    public static function countRows($catid, $statefieldid, $fieldid = null, $fieldvalue = null)
    {

        $app = Factory::getApplication();
        $is_debug =  $app->getUserState('netzsinn.customfieldsfilter.debug');

        $db = Factory::getDbo();
        $query = $db->getQuery(true);
        $query->select('a.id');
        $query->from($db->quoteName('#__content') . ' AS a');
        #$query->join('LEFT', $db->quoteName('#__categories') . ' AS `c` ON `c`.`id` = `a`.`catid`');
        #$query->join('LEFT', $db->quoteName('#__categories') . ' AS `parent` ON `parent`.`id` = `c`.`parent_id`');
        if ($fieldid != null && $fieldvalue != null) {
            $query->join('LEFT', $db->quoteName('#__fields_values', 'fw'), $db->quoteName('fw.item_id') . ' = ' . $db->quoteName('a.id') . ' AND ' . $db->quoteName('fw.field_id') . ' = ' . $tagid);
        }
        $query->join('LEFT', $db->quoteName('#__fields_values', 'fs'), $db->quoteName('fs.item_id') . ' = ' . $db->quoteName('a.id') . ' AND ' . $db->quoteName('fs.field_id') . ' = ' . $statefieldid);

        $query->where($db->quoteName('a.state') . ' = 1');
        $query->where($db->quoteName('a.catid') . ' = :categoryId');
        $query->bind(':categoryId', $catid);
        if ($fieldid != null && $fieldvalue != null) {
            // $query->where($db->quoteName('fw.field_id') . ' = ' . $tagid);
            $query->where($db->quoteName('fw.value') . ' = \'' . $fieldvalue . '\'');
        }
        $query->where($db->quoteName('fs.value') . ' = \'0\'');

        // LEFT JOIN `#__fields_values` AS `fw` ON `fw`.`item_id` = `a`.`id` AND `fw`.`field_id` = 12
        // LEFT JOIN `#__fields_values` AS `fs` ON `fs`.`item_id` = `a`.`id` AND `fs`.`field_id` = 17
        // WHERE `fw`.`value` = '20' AND `fs`.`value` = '0' AND `c`.`published` = 1 AND `a`.`state` = 1 AND `a`.`catid` = :categoryId
        // ORDER BY a.created DESC

        try {
            if ($is_debug == true && !Factory::getUser()->guest) {
                echo '<pre>';
                echo $query;
                echo '</pre>';
            }
            $db->setQuery($query);
            $results = $db->loadObjectList();
            $count = count($results);
            if ($is_debug == true && !Factory::getUser()->guest) {
                echo '<pre>';
                echo 'Treffer: ' . $count;
                echo '</pre>';
            }
        } catch (Exception $e) {
            $msg = $e->getMessage(); // Returns "Normally you would have other code...
            $code = $e->getCode(); // Returns '500';
            Factory::getApplication()->enqueueMessage($msg, 'error'); // commonly to still display that error
            return false;
        }
        return $count;
    }
}
