<?php

/** 
 * Joomla.Plugin - System - NetzSinn Custom Fields Filter
 * ------------------------------------------------------------------------
 * @package     System - NetzSinn Custom Fields Filter
 * @author      Netzsinn.de
 * @copyright   2023 netzsinn.de
 * @license     GNU/GPLv3 <http://www.gnu.org/licenses/gpl-3.0.de.html>
 * @link        https://netzsinn.de
 * ------------------------------------------------------------------------
 */

// no direct access 
defined('_JEXEC') or die;

require_once JPATH_PLUGINS . '/system/netzsinn_customfieldsfilter/customfieldsfilter.helper.php';

use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Router\Route;

/**
 * plg_sys_bw_customfieldsfilter
 */
class PlgSystemNetzsinn_customfieldsfilter extends CMSPlugin
{

	/**
	 * Application object
	 *
	 * @var    \Joomla\CMS\Application\CMSApplication
	 * @since  4.0.0
	 */
	protected $app;

	protected $activate;

	protected $path;

	protected $pathname;

	protected $max_setups = 3;

	protected $settings;

	protected $total;
	/**
	 * Load the language file on instantiation
	 *
	 * @var    boolean
	 * @since  3.1
	 */
	protected $autoloadLanguage = true;

	/** 
	 * This is called before the Model is called. Here we change the input parameters.
	 * 
	 * @see https://docs.joomla.org/Plugin/Events/System#onAfterRoute
	 * @throws Exception
	 */
	public function onAfterRoute()
	{
		if (Factory::getApplication()->isClient('administrator')) {
			return;
		}

		// get path and query params
		$uri = Uri::getInstance();
		$this->path = ltrim($uri->getPath(), '/');

		$input = $this->app->input;
		// $tag = $input->get('tag', null, 'string');
		$option = $input->getString('option', '');

		// Get the settings valid for the given URL ... or leave the Plugin
		$settings = $this->params->get('settings');
		#print_r($settings);
		$this->settings = null;
		foreach ($settings as $setting) {
			$paths = preg_split("/\r\n|\n|\r/", $setting->paths);
			#print_r($paths);
			foreach ($paths as $path) {
				// echo 'current path: ' . $this->path . '<br/>';
				// echo 'path: ' . $path . '<br/>';
				if ($this->path == ltrim($path, '/')) {
					$this->settings = $setting;
					$this->activate = true;
					$this->pathname = ucwords(str_replace('/', '', str_replace('-', ' ', $this->path)));
					break;
				}
			}
		}
		#print_r($this->settings);

		if ($this->settings == null) {
			return true;
		}

		$this->app->setUserState('netzsinn.customfieldsfilter.debug', $this->params->get('is_debug'));
		$this->app->setUserState('netzsinn.customfieldsfilter.maxsetups', $this->max_setups);

		// set basic result data
		$fields = [];
		for ($i = 1; $i <= $this->max_setups; $i++) {
			$field = new stdClass();
			$field->id = $this->settings->{'field' . $i . '_id'};
			$field->value = $this->settings->{'field' . $i . '_value'} == '' ? null : $this->settings->{'field' . $i . '_value'};
			$field->name = '';
			if ($field->value != null && trim($field->value) != '' && str_starts_with($field->value, '?')) {
				$queryparam = ltrim($field->value, '?');
				if (strpos($queryparam, '::') !== false) {
					$parts = explode('::', $queryparam);
					$queryname = $parts[0];
					$queryvalue = $this->app->input->get($queryname, '', 'string');
					$catid = $parts[1];
					$result = NetzSinnCustomfieldsFieldsHelper::resolveTag($queryvalue, $catid);
					$field->value = $result->id;
					$field->name = $result->title;
					$this->pathname = $field->name;
				} else {
					echo 'Error: Please check configuration of ' . $queryparam;
				}
			}
			$fields[$i] = $field;
		}
		$field_order_id = $this->settings->field_order_id;
		$field_order_dir = $this->settings->field_order_dir;

		$this->app->setUserState('netzsinn.customfieldsfilter.fields', $fields);
		$this->app->setUserState('netzsinn.customfieldsfilter.field.order.id', $field_order_id);
		$this->app->setUserState('netzsinn.customfieldsfilter.field.order.dir', $field_order_dir);

		if ($this->params->get('is_debug') == true && !Factory::getUser()->guest) {
			echo '<pre>';
			echo 'path: ' . $this->path . '<br/>';
			echo 'name: ' . $this->pathname . '<br/>';
			echo 'max field definitions: ' . $this->max_setups . '<br/>';
			for ($i = 1; $i <= $this->max_setups; $i++) {
				echo 'field ' . $i . ' id: ' . $fields[$i]->id . '<br/>';
				echo 'field ' . $i . ' value: ' . $fields[$i]->value . '<br/>';
				echo 'field ' . $i . ' name: ' . $fields[$i]->name . '<br/>';
			}
			echo 'field order id: ' . $field_order_id . '<br/>';
			echo 'field order dir: ' . $field_order_dir . '<br/>';
			echo '</pre>';
		}

		if ($this->activate && $option == 'com_content' && !class_exists('ArticlesModel')  && !$this->app->isClient('administrator')) {
			// filter by category or field(s)
			require_once __DIR__ . '/models/com_content/ArticlesModel.php';
		}

		return;
	}

	/**
	 * onAfterRender()
	 *
	 * @return void
	 */
	public function onAfterRender()
	{
		if (Factory::getApplication()->isClient('administrator')) {
			return true;
		}

		if ($this->activate == false) {
			return true;
		}

		// Update HTML Title 
		$title = Factory::getDocument()->getTitle();
		Factory::getDocument()->setTitle($this->pathname);

		$body = $this->app->getBody();

		// Update Total Marker
		if ($this->params->get('items_total') != null &&  strcmp($this->params->get('items_total'), '') != 0) {
			$body = str_replace($this->params->get('items_total'), $this->app->getUserState('netzsinn.customfieldsfilter.items.total'), $body);
		}
		// Update Results per page Marker
		if ($this->params->get('items_perpage') != null &&  strcmp($this->params->get('items_perpage'), '') != 0) {
			$body = str_replace($this->params->get('items_perpage'), $this->app->getUserState('netzsinn.customfieldsfilter.items.perpage'), $body);
		}
		// Update Title marker 
		if ($this->params->get('path_name') != null &&  strcmp($this->params->get('path_name'), '') != 0) {
			$body = str_replace($this->params->get('path_name'), $this->pathname, $body);
		}

		// Update HTML Title 
		if ($this->params->get('items_perpage') == true) {
			#$title = Factory::getDocument()->getTitle();
			Factory::getDocument()->setTitle($this->pathname);
		}

		$this->app->setBody($body);
	}
}
